﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectTractor.Entities;
using ProjectTractor.MovementStrategy;

namespace ProjectTractor.DrawningObjects { 

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
    public class DrawningTractor 
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTractor? EntityTractor { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    protected int _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    protected int _startPosY;
    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    protected readonly int _tractorWidth = 75;
    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    protected readonly int _tractorHeight = 60;
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="width">Ширина картинки</param>
    /// <param name="height">Высота картинки</param>
    public DrawningTractor(int speed, double weight, Color bodyColor, int
    width, int height)
    {        
        if (width <= _tractorWidth || height <= _tractorHeight)
             return;
        _pictureWidth = width;
        _pictureHeight = height;
        EntityTractor = new EntityTractor(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="width">Ширина картинки</param>
    /// <param name="height">Высота картинки</param>
    /// <param name="tractorWidth">Ширина прорисовки автомобиля</param>
    /// <param name="tractorHeight">Высота прорисовки автомобиля</param>
    protected DrawningTractor(int speed, double weight, Color bodyColor, int
    width, int height, int tractorWidth, int tractorHeight)
    {
        if (width <= _tractorWidth || height <= _tractorHeight)
            return;
        _pictureWidth = width;
        _pictureHeight = height;
        _tractorWidth = tractorWidth;
        _tractorHeight = tractorHeight;
        EntityTractor = new EntityTractor(speed, weight, bodyColor);
    }

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningTractor
        /// </summary>
        public IMoveableObject GetMoveableObject => new
        DrawningObjectTractor(this);


        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _tractorWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _tractorHeight;
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанномнаправлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityTractor == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityTractor.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityTractor.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityTractor.Step + _tractorWidth < _pictureWidth,
                // вниз
                DirectionType.Down => _startPosY + EntityTractor.Step + _tractorHeight < _pictureHeight,
                _ => false,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityTractor == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityTractor.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityTractor.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityTractor.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityTractor.Step;
                    break;
            }
        }


        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
    {
            if (EntityTractor == null)
                return;
            _startPosX = x;
            _startPosY = y;
            if (x + _tractorWidth >= _pictureWidth || y + _tractorHeight >= _pictureHeight)
            {
                _startPosX = 1;
                _startPosY = (_tractorHeight + _tractorHeight) / 2;
            }
        }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
            if (EntityTractor == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush brGray = new SolidBrush(Color.Gray);
            Brush brBlack = new SolidBrush(Color.Black);
            Brush additionalBrush = new
            SolidBrush(EntityTractor.BodyColor);

            //Гусеницы         
            g.FillEllipse(brGray, _startPosX, _startPosY + 41, 25, 25);
            g.FillEllipse(brGray, _startPosX + 55, _startPosY + 41, 25, 25);
            g.FillRectangle(brGray, _startPosX + 13, _startPosY + 41, 54, 25);
            //колеса
            g.FillEllipse(brBlack, _startPosX, _startPosY + 45, 15, 15);
            g.FillEllipse(brBlack, _startPosX + 65, _startPosY + 45, 15, 15);
            g.FillEllipse(brBlack, _startPosX + 35, _startPosY + 55, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 20, _startPosY + 55, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 50, _startPosY + 55, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 25, _startPosY + 40, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 45, _startPosY + 40, 10, 10);
            //кузов
            g.FillRectangle(additionalBrush, _startPosX, _startPosY + 20, 80, 20);
            g.FillRectangle(additionalBrush, _startPosX + 60, _startPosY, 10, 20);
            g.FillRectangle(additionalBrush, _startPosX, _startPosY, 40, 20);

            //Окно
            Brush brBlue = new SolidBrush(Color.Blue);
            g.FillRectangle(brBlue, _startPosX + 10, _startPosY + 3, 25, 15);

            //Колеса
            g.FillEllipse(additionalBrush, _startPosX + 2, _startPosY + 47, 11, 11);
            g.FillEllipse(additionalBrush, _startPosX + 67, _startPosY + 47, 11, 11);
            g.FillEllipse(additionalBrush, _startPosX + 37, _startPosY + 57, 6, 6);
            g.FillEllipse(additionalBrush, _startPosX + 22, _startPosY + 57, 6, 6);
            g.FillEllipse(additionalBrush, _startPosX + 52, _startPosY + 57, 6, 6);
            g.FillEllipse(additionalBrush, _startPosX + 27, _startPosY + 42, 6, 6);
            g.FillEllipse(additionalBrush, _startPosX + 47, _startPosY + 42, 6, 6);

            //границы трактора
            g.DrawRectangle(pen, _startPosX, _startPosY + 20, 80, 20);
            g.DrawRectangle(pen, _startPosX + 60, _startPosY, 10, 20);
            g.DrawRectangle(pen, _startPosX, _startPosY, 40, 20);
        }
}
}




